@@ -1,6 +1,6 @@
 from django.test import TestCase
 from django.core.urlresolvers import reverse
-
+from django.shortcuts import get_object_or_404
 from ..forms import ComposeForm, FolderForm
 from ..models import Message, Recipient, Discussion, Folder
 from ..compat import User
@@ -186,20 +186,20 @@ class DiscussionsViewsTests(TestCase):
 
         self.assertEqual(response.status_code, 404)
 
-    def test_valid_discussion_read(self):
-        """ ``POST`` to remove a discussion """
+    def test_valid_discussion_mark_as_read(self):
+        """ ``POST`` to mark a discussion as read """
         # Test that sign in is required
-        response = self.client.post(reverse('discussions_read'))
+        response = self.client.post(reverse('discussions_mark_as_read'))
         self.assertEqual(response.status_code, 302)
 
         # Sign in
         self.client.login(username='thoas', password='$ecret')
 
         # Test that only posts are allowed
-        response = self.client.get(reverse('discussions_read'))
+        response = self.client.get(reverse('discussions_mark_as_read'))
         self.assertEqual(response.status_code, 405)
 
-        response = self.client.post(reverse('discussions_read'),
+        response = self.client.post(reverse('discussions_mark_as_read'),
                                     data={'discussion_ids': '1'})
         self.assertRedirects(response,
                              reverse('discussions_list'))
@@ -208,6 +208,70 @@ class DiscussionsViewsTests(TestCase):
 
         self.assertTrue(recipient.is_read())
 
+    def test_valid_discussion_mark_as_unread(self):
+        """ ``POST`` to mark a discussion as unread"""
+        # Test that sign in is required
+        response = self.client.post(reverse('discussions_mark_as_unread'))
+        self.assertEqual(response.status_code, 302)
+
+        # Sign in
+        self.client.login(username='thoas', password='$ecret')
+
+        # Test that only posts are allowed
+        response = self.client.get(reverse('discussions_mark_as_unread'))
+        self.assertEqual(response.status_code, 405)
+
+        response = self.client.post(reverse('discussions_mark_as_unread'),
+                                    data={'discussion_ids': '1'})
+        self.assertRedirects(response,
+                             reverse('discussions_list'))
+
+        recipient = Discussion.objects.get(pk=1).recipient_set.get(user=User.objects.get(pk=1))
+
+        self.assertFalse(recipient.is_read())
+
+    def test_valid_discussion_leave(self):
+        """ ``POST`` to leave a discussion
+        # Test that sign in is required'"""
+        response = self.client.post(reverse('discussions_leave'))
+        self.assertEqual(response.status_code, 302)
+
+        # Sign in
+        self.client.login(username='ampelmann', password='$ecret')
+
+        # Test that only posts are allowed
+        response = self.client.get(reverse('discussions_leave'))
+        self.assertEqual(response.status_code, 405)
+
+        response = self.client.post(reverse('discussions_leave'),
+                                    data={'discussion_ids': [1]})
+
+        self.assertRedirects(response,
+                             reverse('discussions_list'))
+
+        self.assertFalse(Discussion.objects.get(pk=1).recipients.filter(pk=2).exists())
+
+    def test_invalid_discussion_leave_when_sender(self):
+        """ ``POST`` to leave a discussion
+        # Test that sign in is required'"""
+        response = self.client.post(reverse('discussions_leave'))
+        self.assertEqual(response.status_code, 302)
+
+        # Sign in
+        self.client.login(username='thoas', password='$ecret')
+
+        # Test that only posts are allowed
+        response = self.client.get(reverse('discussions_leave'))
+        self.assertEqual(response.status_code, 405)
+
+        response = self.client.post(reverse('discussions_leave'),
+                                    data={'discussion_ids': [1]})
+
+        self.assertRedirects(response,
+                             reverse('discussions_list'))
+
+        self.assertTrue(Discussion.objects.get(pk=1).recipients.filter(pk=2).exists())
+
     def test_valid_discussion_remove(self):
         """ ``POST`` to remove a discussion """
         # Test that sign in is required
@@ -327,14 +391,22 @@ class DiscussionsViewsTests(TestCase):
 
         self.assertTemplateUsed(response, 'discussions/unread.html')
 
+    def test_discussion_read(self):
+        """ ``GET`` the message list for a user """
+        self._test_login("discussions_unread")
+
+        self.client.login(username='thoas', password='$ecret')
+        response = self.client.get(reverse('discussions_read'))
+        self.assertEqual(response.status_code, 200)
+
+        self.assertTemplateUsed(response, 'discussions/read.html')
+
     def test_discussion_detail_between_two_users(self):
         """ ``GET`` to a detail page between two users """
-        self._test_login('discussions_list',
-                         kwargs={'username': 'oleiade'})
+        self._test_login('discussions_list')
         self.client.login(username='thoas', password='$ecret')
 
-        response = self.client.get(reverse('discussions_list',
-                                           kwargs={'username': 'oleiade'}))
+        response = self.client.get(reverse('discussions_list'))
 
         self.assertEqual(response.status_code, 200)
 
@@ -349,6 +421,16 @@ class DiscussionsViewsTests(TestCase):
 
         self.assertEqual(len(unread_messages), 0)
 
+    def test_folder_list(self):
+        """ ``GET`` the discussion list for a user """
+        self._test_login("discussions_folders_list")
+
+        self.client.login(username='thoas', password='$ecret')
+        response = self.client.get(reverse('discussions_folders_list'))
+        self.assertEqual(response.status_code, 200)
+
+        self.assertTemplateUsed(response, 'discussions/folder/list.html')
+
     def test_folder_create(self):
         self._test_login('discussions_folder_create')
 
@@ -375,9 +457,7 @@ class DiscussionsViewsTests(TestCase):
         self.assertEqual(folder.name, data['name'])
 
         self.assertRedirects(response,
-                             reverse('discussions_folder_detail', kwargs={
-                                 'folder_id': folder.pk
-                             }))
+                             reverse('discussions_list'))
 
     def test_folder_update(self):
         folder = Folder.objects.get(pk=1)
@@ -411,6 +491,85 @@ class DiscussionsViewsTests(TestCase):
         self.assertEqual(folder.name, data['name'])
 
         self.assertRedirects(response,
-                             reverse('discussions_folder_update', kwargs={
+                             reverse('discussions_list', kwargs={
                                  'folder_id': folder.pk
                              }))
+
+    def test_discussion_move_to_folder(self):
+        response = self.client.post(reverse('discussions_move',
+                                            kwargs={'folder_id': 1}),
+                                    data={'discussion_ids': [2]})
+        self.assertEqual(response.status_code, 302)
+
+        # Sign in
+        self.client.login(username='thoas', password='$ecret')
+
+        # Test that only posts are allowed
+        response = self.client.get(reverse('discussions_move',
+                                           kwargs={'folder_id': 1}),
+                                   data={'discussion_ids': [2]})
+        self.assertEqual(response.status_code, 405)
+
+        response = self.client.post(reverse('discussions_move',
+                                            kwargs={'folder_id': 1}),
+                                    data={'discussion_ids': [2]})
+        recipient = Recipient.objects.get(pk=2)
+
+        folder = Folder.objects.get(pk=1)
+
+        self.assertEqual(recipient.folder, folder)
+
+    def test_discussion_move_to_mailbox(self):
+        response = self.client.post(reverse('discussions_move'),
+                                    data={'discussion_ids': [3]})
+        self.assertEqual(response.status_code, 302)
+
+        # Sign in
+        self.client.login(username='thoas', password='$ecret')
+
+        # Test that only posts are allowed
+        response = self.client.get(reverse('discussions_move'),
+                                   data={'discussion_ids': [3]})
+        self.assertEqual(response.status_code, 405)
+
+        response = self.client.post(reverse('discussions_move'),
+                                    data={'discussion_ids': [3]})
+        recipient = Recipient.objects.get(pk=3)
+
+        self.assertEqual(recipient.folder, None)
+
+    def test_folder_remove_pk_does_not_exist(self):
+        # Sign in
+        self.client.login(username='thoas', password='$ecret')
+
+        response = self.client.post(reverse('discussions_folder_remove',
+                                    kwargs={'folder_id': 35}))
+
+        self.assertEqual(response.status_code, 404)
+
+    def test_valid_folder_remove(self):
+        response = self.client.post(reverse('discussions_folder_remove',
+                                            kwargs={'folder_id': 1}))
+        self.assertEqual(response.status_code, 302)
+
+        # Sign in
+        self.client.login(username='thoas', password='$ecret')
+
+        # Test that only posts are allowed
+        response = self.client.get(reverse('discussions_folder_remove',
+                                           kwargs={'folder_id': 1}))
+        self.assertEqual(response.status_code, 405)
+
+        folder = Folder.objects.get(pk=1)
+        related_recipient = Recipient.objects.filter(folder=folder)
+
+        response = self.client.post(reverse('discussions_folder_remove',
+                                            kwargs={'folder_id': 1}))
+
+        for recipient in related_recipient:
+            self.assertEqual(recipient.folder, None)
+
+        self.assertRedirects(response,
+                             reverse('discussions_list'))
+
+        self.assertFalse(Folder.objects.filter(pk=1).exists())
